name: "genesis_notification"
description: "Genesis notification element"
schema_version: 1
version: "{{ version }}"
api_version: "v1"

requirements:
  core:
    from_version: "0.0.0"

resources:
  $core.compute.sets:
    genesis_notification_set:
      name: "genesis-notification"
      project_id: "12345678-c625-4fee-81d5-f691897b8142"
      cores: 1
      ram: 1024
      replicas: 1
      disk_spec:
        kind: "root_disk"
        size: 10
        image: "{{ repository | default('https://repository.genesis-core.tech/genesis-elements') }}/genesis_notification/{{ version }}/images/genesis-notification.raw.gz"
